local ModListWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModListWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


--commonapi._reloadModule("_modmm")
local modmm = commonapi._modmm;

local ModSettingsWindow = require("commonapi2.uicommon.ModSettingsWindow")

ModListWindow = ui.Window:new()

ModListWindow.id = "ModListWindow"
ModListWindow.height = 500
ModListWindow.width = 600


ModListWindow.caption = _("Mod settings")
ModListWindow.data = {}

function ModListWindow.create()
	local o
	o = ModListWindow:new()
	o:genModUIList()
	o:open()
	return o
end


function ModListWindow:refresh()
	self:genModUIList()
end


function ModListWindow:selectModForInstall()
	local window = self
	local FileDialog = require("commonapi2.ui.FileDialog")
	local fd = FileDialog.createOpen()
	fd.id = "FileDialog_ModInstall_Select"
	fd.caption = _("Install mod")

	local modinstallpath = commonapi._native.paths.getUserPath()
	if (modinstallpath ~= nil) then
		--modinstallpath = userpath .. "modlists"
		--commonapi._native.fs.mkdir(modlistpath)
	else
		modinstallpath = commonapi._info.path
	end

	fd.path = modinstallpath
	fd.fileext = {".zip", ".rar", ".7z", ".package"}
	fd.loadFile = function(self, filename)
		self:close()
		ModListWindow.installMod(window, filename)
	end
	fd:open()
end
function ModListWindow:installMod(filename)
	--commonapi.dmp(filename)
	--modmm.getFileInfo(filename)

	local ok, errormsg = modmm.installModFromArchiv(filename)
	if (ok == false) then
		ui.msgbox('ModListWindow:installMod', "Fehler beim installieren: \n"..errormsg, "Install Mod")
	end

end

function ModListWindow:genDepWindow(modid)
	local uilist = {}


	--local modentry = commonapi.mods.getById(modid)
	local modentry = commonapi.mods.getById(modid)
	if (modentry == nil) then
		modentry = commonapi.mods._getByIdAny(modid)
		if (modentry == nil) then
			return
		end
	end

	local requiredMods = modentry.data.info.requiredMods or {}

	local depwin = ui.Window:new()

	depwin.id = "ModDepWindow"
	depwin.caption = _("Mod dependencies:")

	for m, entry in pairs( requiredMods ) do

		local links = commonapi.mods._checkModDepends_createLinks(entry)

		local uimodid = {
			typ = "Label",
			caption = ""..(entry.modId or "No modId").." , "..(entry.steamId or "No SteamId")
		}
		uilist[#uilist+1] = uimodid
		local uiversion = {
			typ = "Label",
			caption = "\tMinorVersion: >="..(entry.minMinorVersion or 0)
		}
		uilist[#uilist+1] = uiversion

		for urltype, url in pairs( links ) do
			local urlentry = {
				typ = "Button",
				caption = urltype,
				sameline = true,
				id = ""..m..''..urltype,
				data = {
					myurl = url,
				},
				onclick = function(item, win)
					commonapi._native.utils.openurl(item.data.myurl)
				end
			}
			uilist[#uilist+1] = urlentry
		end
	end
	depwin.uilist = uilist

	depwin:open()
end



function ModListWindow:genDepList(uilist, modentry)
	local requiredMods = modentry.data.info.requiredMods or {}

	for m, entry in pairs( requiredMods ) do

		local links = commonapi.mods._checkModDepends_createLinks(entry)

		local uimodid = {
			typ = "Label",
			caption = ""..(entry.modId or "No modId").." , "..(entry.steamId or "No SteamId")
		}
		uilist[#uilist+1] = uimodid
		local uiversion = {
			typ = "Label",
			caption = "\tMinorVersion: >="..(entry.minMinorVersion or 0)
		}
		uilist[#uilist+1] = uiversion

		for urltype, url in pairs( links ) do
			local urlentry = {
				typ = "Button",
				caption = urltype,
				sameline = true,
				id = ""..m..''..urltype,
				data = {
					myurl = url,
				},
				onclick = function(item, win)
					commonapi._native.utils.openurl(item.data.myurl)
				end
			}
			uilist[#uilist+1] = urlentry
		end
	end
end


function ModListWindow:genUpgradeList(uilist, entry)
	for mu, repoentry in ipairs( entry.repomods ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end
		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = entry.id..'__repo__'..mu,
			caption = caption,
			data = {
				modid = entry.id,
				repoentry = repoentry,
				--repodata = entry.repodata
			},
			onclick = function(item, win)
				--entry.downloadfull = repodata.download_prefix_url..''..entry.download
				--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
				--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..entry.id.." \n"..errormsg, "Update")
							end
						end
					},
				})
			end,
		}
	end
end


function ModListWindow:genModDetails(entry)
	if (self.uilist[#self.uilist].typ == "TabBar") then
		self.uilist[#self.uilist] = nil
	end
	local uiTabs = {}

	local uiTabBar = {
		typ = "TabBar",
		id = "ModDetailsTab",
		uilist = uiTabs
	}

	local uilistInfo ={}

	uilistInfo[#uilistInfo+1] = {
		typ = "Label",
		caption = entry.name,
	}
	uilistInfo[#uilistInfo+1] = {
		typ = "Label",
		caption = table.concat(entry.version, "."),
	}


	uiTabs[#uiTabs+1] = {
		typ = "TabItem",
		id = "ModDetailsTabInfo",
		caption = "Details",
		uilist = uilistInfo,
	}

	local uilistDesc = {}
	if (entry.data ~= nil and entry.data.info ~= nil) then
		uilistDesc[#uilistDesc+1] = {
			typ = "Text",
			text = entry.data.info.description or "<No Description>",
		}
	end
	uiTabs[#uiTabs+1] = {
		typ = "TabItem",
		id = "ModDetailsTabDesc",
		caption = _("Description"),
		uilist = {
			{
				typ = "ScrollRegion",
				id = "ScrollRegion",
				size = {
					0, 0,
				},
				uilist = uilistDesc,
			}
		},
	}

	local requiredMods = entry.data.info.requiredMods or {}

	if (#requiredMods > 0) then
		local uilistDeps = {}
		uiTabs[#uiTabs+1] = {
			typ = "TabItem",
			id = "ModDetailsTabDep",
			caption = _("Dependencies"),
			uilist = uilistDeps,
		}
		self:genDepList(uilistDeps, entry)
	end

	if (entry.repomods ~= nil) then
		local uilistUpgrade = {}
		uiTabs[#uiTabs+1] = {
			typ = "TabItem",
			id = "ModDetailsTabUpgrade",
			caption = _("Upgrade"),
			uilist = {
				{
					typ = "ScrollRegion",
					id = "ScrollRegion",
					size = {
						0, 0,
					},
					uilist = uilistUpgrade,
				}
			},
		}
		self:genUpgradeList(uilistUpgrade, entry)
	end

	self.uilist[#self.uilist+1] =  uiTabBar
end

function ModListWindow:genModUIList_CreateRow(uilist, entry)
	if (entry.data ~= nil and entry.data.info ~= nil) then
		if (entry.data.info.visible == false) then
			return
		end
	end

	local hasUpgrade = (entry.upgradeversion ~= nil)
	local uientry = {
		--typ = "Label",
		typ = "_Selectable",
		_fix_overlap = true,
		id = entry.id..'__label',
		caption = entry.name .. '  '..entry.id,
		column = 0,
		--columnwidth = 200.0,
		rownew = true,
		onclick = function(item, win)
			win.data['curentry'] = entry
			win:genModDetails(entry)
		end,
		selected = function(item, win)
			return (win.data['curentry'] == entry)
		end
	}

	if (hasUpgrade) then
		uientry.color = { 1.0, 1.0, 0, 1.0 }
	end
	uilist[#uilist+1] = uientry

	if (hasUpgrade) then
		local uientryPopup = {
			typ = "SmallButton",
			id = entry.id..'_dopopup',
			caption = _("Upgrade"),
			sameline = true, 
			onclick = function(item, win)
				win.params[entry.id..'_repo_version'] = true
			end
		}
		uilist[#uilist+1] = uientryPopup
	end
	local uientryVersion = {
		typ = "Label",
		caption = table.concat(entry.version, "."),
		column = 1,
		--columnoffset = 250.0,
	}
	if (hasUpgrade) then
		uientryVersion.caption = uientryVersion.caption .. ' - ' .. table.concat(entry.upgradeversion, ".")
	end
	uilist[#uilist+1] = uientryVersion


	local popupList = {
		{
			typ = "Label",
			caption = "Click for Download"
		},
		{
			typ = "Separator",
		}
	}

	if (entry.repomods ~= nil) then
		self:genUpgradeList(popupList, entry)

		local popupRepo = {
			typ = "_Popup",
			id = entry.id..'_repo_version',
			uilist = popupList
		}
		uilist[#uilist+1] = popupRepo
	end

	local uientryButton = {
		typ = "SmallButton",
		id = entry.id..'_settings',
		caption = _("Settings"),
		column = 2,
		--columnoffset = 300.0,
		--columnoffset = self.width-60.0-100.0,
		--columnwidth = 100,
		data = {
			modid = entry.id,
		},
		onclick = function(item, win)
			local o = ModSettingsWindow.create(item.data.modid)
			o:open()
		end
	}

	if (commonapi.settings._getDescription(entry.id) ~= nil) then
		uilist[#uilist+1] = uientryButton
	end

--[[	
	if (entry.data.info ~= nil and entry.data.info.requiredMods ~= nil) then
		local uidepButton = {
			typ = "SmallButton",
			id = entry.id..'_dep',
			caption = _("Dep"),
			column = 3,
			--columnoffset = self.width-60.0,
			--sameline = true,
			data = {
				modid = entry.id,
			},
			onclick = function(item, win)
				win:genDepWindow(item.data.modid)
			end
		}
		uilist[#uilist+1] = uidepButton
	end
--]]
end

function ModListWindow:genModUIList()
	local uilist = {}
	
	local mods
	if (self.params["showallmods"] == true) then
		mods = commonapi.mods._getLoadedNotLoaded()
	else
		mods = commonapi.mods.getLoaded()
	end

	local timestamp = modmm.applyRepositoryFilename("tpfnet-tpf2.json", mods)
	if (timestamp ~= nil) then
		self.data.repotimestamp = timestamp
	else
		self.data.repotimestamp = nil
	end


	uilist[#uilist+1] = {
		typ = "Checkbox",
		id = "showallmods",
		caption = _("Show all mods"),
		onclick = function(item, win)
			win:refresh()
		end
	}

	uilist[#uilist+1] = {
		typ = "SmallButton",
		id = "installMod",
		caption = _("Install mod"),
		sameline = true,
		onclick = function(item, win)
			win:selectModForInstall()
		end
	}

	uilist[#uilist+1] = {
		typ = "SmallButton",
		id = "checkUpdates",
		caption = _("Download update information"),
		sameline = true,
		onclick = function(item, win)
			win:checkUpdates()
		end
	}

	if (self.data.repotimestamp ~= nil) then
		uilist[#uilist+1] = {
			typ = "Label",
			caption = os.date('%Y-%m-%d %H:%M:%S', self.data.repotimestamp),
			sameline = true,
		}
	end

	local uiScrollList = {}
	uilist[#uilist+1] = {
		typ = "ScrollRegion",
		id = "ScrollRegion",
		border = false,
		size = {
			0, -100
		},
		uilist = uiScrollList
	}
	
	local uicolumns = {
		typ = "Columns",
		count = 3,
		border = true,
	}
	uiScrollList[#uiScrollList+1] = uicolumns

--[[
	local columns = {
		{ "Name", 0 },
		{ "Version", self.width-70-100-60 },
		{ _("Settings"), self.width-70-100 },
		{ "Dep", self.width-70 },
	}
--]]
	local columns = {
		{ "Name", 0 },
		{ "Version", self.width-120-60 },
		{ _("Settings"), self.width-120 }
	}

	for i, column in ipairs(columns) do
		uiScrollList[#uiScrollList+1] = {
			typ = "Label",
			column = i-1,
			caption = column[1],
			columnoffset = column[2],
		}
	end

	for m, entry in pairs( mods ) do
		self:genModUIList_CreateRow(uiScrollList, entry)
	end
	uilist[#uilist+1] = {
		typ = "Separator"
	}
	self.uilist = uilist
end



function ModListWindow:checkUpdates()
	local repositoryurl = "https://www.transportfever.net/filebase/repos/tpf2.json"
	local ok, errormsg = modmm.downloadRepository("tpfnet-tpf2.json", repositoryurl)
	if (ok == false) then
		ui.msgbox('ModListWindow_checkUpdates', "Fehler beim Updaten: \n"..errormsg, "Download repository")
	end

	self:refresh()
end

return ModListWindow